/*************************************************************************

Copyright (c) 2016 Novell, Inc.  All Rights Reserved.

With respect to this file, Novell hereby grants to Developer a
royalty-free, non-exclusive license to include this sample code
and derivative binaries in its product. Novell grants to Developer
worldwide distribution rights to market, distribute or sell this
sample code file and derivative binaries as a component of
Developer's product(s).  Novell shall have no obligations to
Developer or Developer's customers with respect to this code.

DISCLAIMER:

Novell disclaims and excludes any and all express, implied, and
statutory warranties, including, without limitation, warranties
of good title, warranties against infringement, and the implied
warranties of merchantibility and fitness for a particular purpose.
Novell does not warrant that the software will satisfy customer's
requirements or that the licensed works are without defect or error
or that the operation of the software will be uninterrupted.
Novell makes no warranties respecting any technical services or
support tools provided under the agreement, and disclaims all other
warranties, including the implied warranties of merchantability and
fitness for a particular purpose. */

/**************************************************************************
getDirSpaceLimit.c
**************************************************************************/ 

#include <stdlib.h>
#include <stdio.h>
#include <dirent.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>

#include <nwcalls.h>
#include <nwclxcon.h>
#include <nwnet.h>
#include <nwerror.h>

#define XPLAT_NO_QUOTA 0x7FFFFFFFFFFFFFFFULL

int main(int argc, char **argv)
{
	//userObject is used just to authenticate to the tree                                
	char nwPath[1024], userObject[MAX_DN_CHARS], password[32], server[NW_MAX_SERVER_NAME_LEN];
	NWRCODE ccode;
	NWCONN_HANDLE conn;
	NWDSContextHandle ctxHandle;
	char treeName[NW_MAX_TREE_NAME_LEN];

	if (argc == 5)
	{
		strcpy(server, argv[1]);
		strcpy(userObject, argv[2]);
		strcpy(password, argv[3]);
		strcpy(nwPath, argv[4]);
	}
	else
	{
		printf("Usage:\n  getDirSpaceLimit <server> <userObject> <password> <nwPath>\n"); 
		exit(1);
	}

	ccode = NWCallsInit(NULL,NULL);

	if (ccode != SUCCESS)
	{
		printf("NWCallsInit failed with code 0x%04x\n", ccode);
		exit(1);
	}

	ccode = NWCCOpenConnByName (0, server, NWCC_NAME_FORMAT_BIND, NWCC_OPEN_LICENSED, NWCC_TRAN_TYPE_WILD, &conn);
	if (ccode == SUCCESS)
	{
		printf("Opened connection\n");

		ccode = NWCCGetConnInfo(conn, NWCC_INFO_TREE_NAME, sizeof(treeName), treeName);
		if (ccode == SUCCESS )
		{
			printf("   Tree name: %s\n", treeName);

			ccode = NWDSCreateContextHandle(&ctxHandle);
			if (ccode == SUCCESS)
			{
				// There must be a tree name in the context handle or a default tree name
				NWDSSetContext(ctxHandle, DCK_TREE_NAME, treeName);

				if (NWDSCanDSAuthenticate(ctxHandle) != TRUE)
				{
					NWDSSetContext(ctxHandle, DCK_NAME_CONTEXT, "");

					ccode = NWDSLoginEx(ctxHandle, userObject, PWD_RAW_C_STRING, password);
					if (ccode == SUCCESS || ccode == NWE_ALREADY_ATTACHED)
					{
						printf("   Logged in\n");
					}
					else
						printf("NWDSLoginEx() returned 0x%x (%d)\n", ccode, ccode);
				}
				else
				{
					ccode = NWDSAuthenticateConn(ctxHandle, conn);
					if (ccode)
						printf("NWDSAuthenticateConn() returned 0x%x\n", ccode);
				}

				NWDSFreeContext(ctxHandle);
			}
			else
				printf("NWDSCreateContextHandle() returned 0x%x (%d)\n", ccode, ccode);
		}
		else
			printf("NWCCGetConnInfo() returned 0x%x (%d)\n", ccode, ccode);
	}
	else
		printf("NWCCOpenConnByName() returned 0x%x (%d)\n", ccode, ccode);

	if (ccode)
		exit(1);

	nuint64 ui64Restriction = 0; // Default return value.
	nuint64 ui64SpaceAvailable = 0;
	NWDIR_HANDLE dirHandle = 0;
	NWCONN_HANDLE newConnID = 0;

	ccode = NWAllocTempNSDirHandle3(conn, 0, nwPath,NW_NS_LONG,&newConnID, &dirHandle, NW_NS_LONG);

	if ( !ccode )
	{
		// If parsing the path led us to different server (DFS junction), use different server conn.
		if ( newConnID ) 
			conn = newConnID;

		NW_RESTRICTION_64 Restriction;	
		memset( &Restriction,0, sizeof( Restriction ) );

		ccode = NWGetDirSpaceLimit( conn, dirHandle, &Restriction);

		if(ccode)
		{
			printf("NWGetDirSpaceLimit failed, ccode = 0x%x\n", ccode);
			return ccode;
		}

		if ( Restriction.ui64SpaceRestriction == XPLAT_NO_QUOTA )
		{
			ui64Restriction = XPLAT_NO_QUOTA;
			printf("There is no directory Space Restriction on directory \"%s\" \n",nwPath);
		}
		else if ( Restriction.ui64SpaceRestriction == 0 )
		{
			ui64Restriction = 0;
			printf("File/Folder creation is disabled on directory \"%s\" \n",nwPath);
		}
		else
		{
			// Restrinction units are expressed in 4KB blocks.  Convert to BYTES, not kilobytes.
			ui64Restriction = Restriction.ui64SpaceRestriction * 4096; 
			ui64SpaceAvailable = Restriction.ui64SpaceAvailable * 4096;
			printf("The Space Restriction on directory \"%s\" is %lu bytes \n", nwPath,ui64Restriction);
			printf("The available free space on directory \"%s\" is %lu bytes \n",nwPath, ui64SpaceAvailable);
		}

		ccode = NWDeallocateDirectoryHandle(conn, dirHandle);
		if (ccode)
		{
			printf("NWDeallocateDirectoryHandle() returned 0x%x (%d)\n", ccode, ccode);
		}

		// Don't close connID, but do close newConnID if one was opened.
		if ( newConnID ) 
			NWCCCloseConn( newConnID );
	}
	else
	{
		printf("NWAllocTempNSDirHandle3 failed, ccode = 0x%x\n", ccode);
		return ccode;
	}


	NWDSLogout(ctxHandle);
	NWDSFreeContext(ctxHandle);
	NWCCCloseConn(conn);
	NWCallsTerm(NULL);
	return(0);
}


