/*************************************************************************

Copyright (c) 2016 Novell, Inc.  All Rights Reserved.

With respect to this file, Novell hereby grants to Developer a
royalty-free, non-exclusive license to include this sample code
and derivative binaries in its product. Novell grants to Developer
worldwide distribution rights to market, distribute or sell this
sample code file and derivative binaries as a component of
Developer's product(s).  Novell shall have no obligations to
Developer or Developer's customers with respect to this code.

DISCLAIMER:

Novell disclaims and excludes any and all express, implied, and
statutory warranties, including, without limitation, warranties
of good title, warranties against infringement, and the implied
warranties of merchantibility and fitness for a particular purpose.
Novell does not warrant that the software will satisfy customer's
requirements or that the licensed works are without defect or error
or that the operation of the software will be uninterrupted.
Novell makes no warranties respecting any technical services or
support tools provided under the agreement, and disclaims all other
warranties, including the implied warranties of merchantability and
fitness for a particular purpose. */

/**************************************************************************
getObjDiskRestrictionsExt.c
**************************************************************************/ 

#include <stdlib.h>
#include <stdio.h>
#include <dirent.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>

#include <nwcalls.h>
#include <nwclxcon.h>
#include <nwnet.h>
#include <nwerror.h>

#define XPLAT_NO_QUOTA 0x7FFFFFFFFFFFFFFFULL

int main(int argc, char **argv)
{                                
	//userObject is used to authenticate to the tree, and to query user quota on a volume
	char volName[16], userObject[MAX_DN_CHARS], password[32], server[NW_MAX_SERVER_NAME_LEN];
	NWRCODE ccode;
	NWCONN_HANDLE conn;
	NWDSContextHandle ctxHandle;
	char treeName[NW_MAX_TREE_NAME_LEN];

	if (argc == 5)
	{
		strcpy(server, argv[1]);
		strcpy(userObject, argv[2]);
		strcpy(password, argv[3]);
		strcpy(volName, argv[4]);
	}
	else
	{
		printf("Usage:\n  getObjDiskRestrictions <server> <userObject> <password> <volName>\n");
		exit(1);
	}

	ccode = NWCallsInit(NULL,NULL);

	if (ccode != SUCCESS)
	{
		printf("NWCallsInit failed with code 0x%04x\n", ccode);
		exit(1);
	}

	ccode = NWCCOpenConnByName (0, server, NWCC_NAME_FORMAT_BIND, NWCC_OPEN_LICENSED, NWCC_TRAN_TYPE_WILD, &conn);
	if (ccode == SUCCESS)
	{
		printf("Opened connection\n");

		ccode = NWCCGetConnInfo(conn, NWCC_INFO_TREE_NAME, sizeof(treeName), treeName);
		if (ccode == SUCCESS )
		{
			printf("   Tree name: %s\n", treeName);

			ccode = NWDSCreateContextHandle(&ctxHandle);
			if (ccode == SUCCESS)
			{
				// There must be a tree name in the context handle or a default tree name
				NWDSSetContext(ctxHandle, DCK_TREE_NAME, treeName);

				if (NWDSCanDSAuthenticate(ctxHandle) != TRUE)
				{
					NWDSSetContext(ctxHandle, DCK_NAME_CONTEXT, "");

					ccode = NWDSLoginEx(ctxHandle, userObject, PWD_RAW_C_STRING, password);
					if (ccode == SUCCESS || ccode == NWE_ALREADY_ATTACHED)
					{
						printf("   Logged in\n");
					}
					else
						printf("NWDSLoginEx() returned 0x%x (%d)\n", ccode, ccode);
				}
				else
				{
					ccode = NWDSAuthenticateConn(ctxHandle, conn);
					if (ccode)
						printf("NWDSAuthenticateConn() returned 0x%x\n", ccode);
				}

			}
			else
				printf("NWDSCreateContextHandle() returned 0x%x (%d)\n", ccode, ccode);
		}
		else
			printf("NWCCGetConnInfo() returned 0x%x (%d)\n", ccode, ccode);
	}
	else
		printf("NWCCOpenConnByName() returned 0x%x (%d)\n", ccode, ccode);

	if (ccode)
		exit(1);

	nuint16  volNumber;

	ccode = NWGetVolumeNumber(conn, volName, &volNumber);

	if(ccode)
	{
		printf("NWGetVolumeNumber for volume \"%s\" on connectionID 0x%08X returned status 0x%08X.\n", volName, conn, ccode );                                 
		return (ccode);
	}

	nuint32 userID = 0;
	nuint64 restriction = 0, inUse = 0;
	nuint64 currentDiskUse = 0, maxDiskUse = 0;
	nuint32 volNum=volNumber;	

	ccode = NWDSMapNameToID( ctxHandle, conn, userObject, &userID );

	if(ccode)
	{
		printf("NWDSMapNameToID() returned 0x%x (%d)\n", ccode, ccode);
		return ccode;
	}
	else 
		printf("Opened connection to userID %u \n", userID);

	ccode = NWGetObjDiskRestrictionsExt( conn, volNum, userID, &restriction, &inUse );

	if(ccode)
	{
		printf("NWGetObjDiskRestrictionsExt for volNum %u on connectionID 0x%08X failed. Status=0x%08X.\n", volNum, conn, ccode );
		return (ccode);
	}

	// Per NWGetObjDiskRestrictions documentation, "The restrictions are returned in units of 4KB
	// blocks and ignore the block size of the volume."

	currentDiskUse = inUse * 4 * 1024;

	if ( restriction == XPLAT_NO_QUOTA )
		printf("There is no disk space restrictions on volume \"%s\" for user object \"%s\".Currently used space is %lu bytes \n",volName,userObject,currentDiskUse);
	else if ( restriction == 0 )
		printf("File/Folder creation is disabled on volume \"%s\" for user object \"%s\".Currently usedspace is %lu bytes\n",volName,userObject,currentDiskUse);
	else
	{
		// Per NWGetObjDiskRestrictions documentation, "The restrictions are returned in units of 4KB 
		// blocks and ignore the block size of the volume."

		maxDiskUse = restriction * 4 * 1024; 

		printf("Space restrictions on volume \"%s\" for user object \"%s\" is %lu bytes.Currently used space is %lu bytes\n",volName,userObject,maxDiskUse,currentDiskUse);  
	}

	NWDSLogout(ctxHandle);
	NWDSFreeContext(ctxHandle);
	NWCCCloseConn(conn);
	NWCallsTerm(NULL);
	return(0);
}



