/*************************************************************************

Copyright (c) 2016 Novell, Inc.  All Rights Reserved.

With respect to this file, Novell hereby grants to Developer a
royalty-free, non-exclusive license to include this sample code
and derivative binaries in its product. Novell grants to Developer
worldwide distribution rights to market, distribute or sell this
sample code file and derivative binaries as a component of
Developer's product(s).  Novell shall have no obligations to
Developer or Developer's customers with respect to this code.

DISCLAIMER:

Novell disclaims and excludes any and all express, implied, and
statutory warranties, including, without limitation, warranties
of good title, warranties against infringement, and the implied
warranties of merchantibility and fitness for a particular purpose.
Novell does not warrant that the software will satisfy customer's
requirements or that the licensed works are without defect or error
or that the operation of the software will be uninterrupted.
Novell makes no warranties respecting any technical services or
support tools provided under the agreement, and disclaims all other
warranties, including the implied warranties of merchantability and
fitness for a particular purpose. */

/**************************************************************************
getVolumeDetailsByInfoMask.c
**************************************************************************/ 

#include <stdlib.h>
#include <stdio.h>
#include <dirent.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>

#include <nwcalls.h>
#include <nwclxcon.h>
#include <nwnet.h>
#include <nwerror.h>

#define VINFO_RIM_VOL_INFO64   0x00000001
#define VINFO_RIM_VOL_NAME     0x00000002

int main(int argc, char **argv)
{                                
	//userObject is used just to authenticate to the tree and not to
	//query user quota on a volume
	char volName[16], userObject[MAX_DN_CHARS], password[32], server[NW_MAX_SERVER_NAME_LEN];
	NWRCODE ccode;
	NWCONN_HANDLE conn;
	NWDSContextHandle ctxHandle;
	char treeName[NW_MAX_TREE_NAME_LEN];

	if (argc == 6)
	{
		strcpy(server, argv[1]);
		strcpy(userObject, argv[2]);
		strcpy(password, argv[3]);
		strcpy(volName, argv[4]);
	}
	else
	{
		printf("Usage:\n  getVolumeDetailsByInfoMask <server> <userObject> <password> <volName> <1|2|3> \n   1 - Volume Statistics \n   2 - Volume Name Details \n   3 - All Volume Info \n");
		exit(1);
	}

	ccode = NWCallsInit(NULL,NULL);

	if (ccode != SUCCESS)
	{
		printf("NWCallsInit failed with code 0x%04x\n", ccode);
		exit(1);
	}

	ccode = NWCCOpenConnByName (0, server, NWCC_NAME_FORMAT_BIND, NWCC_OPEN_LICENSED, NWCC_TRAN_TYPE_WILD, &conn);
	if (ccode == SUCCESS)
	{
		printf("Opened connection\n");

		ccode = NWCCGetConnInfo(conn, NWCC_INFO_TREE_NAME, sizeof(treeName), treeName);
		if (ccode == SUCCESS )
		{
			printf("   Tree name: %s\n", treeName);

			ccode = NWDSCreateContextHandle(&ctxHandle);
			if (ccode == SUCCESS)
			{
				// There must be a tree name in the context handle or a default tree name
				NWDSSetContext(ctxHandle, DCK_TREE_NAME, treeName);

				if (NWDSCanDSAuthenticate(ctxHandle) != TRUE)
				{
					NWDSSetContext(ctxHandle, DCK_NAME_CONTEXT, "");

					ccode = NWDSLoginEx(ctxHandle, userObject, PWD_RAW_C_STRING, password);
					if (ccode == SUCCESS || ccode == NWE_ALREADY_ATTACHED)
					{
						printf("   Logged in\n");
					}
					else
						printf("NWDSLoginEx() returned 0x%x (%d)\n", ccode, ccode);
				}
				else
				{
					ccode = NWDSAuthenticateConn(ctxHandle, conn);
					if (ccode)
						printf("NWDSAuthenticateConn() returned 0x%x\n", ccode);
				}

				NWDSFreeContext(ctxHandle);
			}
			else
				printf("NWDSCreateContextHandle() returned 0x%x (%d)\n", ccode, ccode);
		}
		else
			printf("NWCCGetConnInfo() returned 0x%x (%d)\n", ccode, ccode);
	}
	else
		printf("NWCCOpenConnByName() returned 0x%x (%d)\n", ccode, ccode);

	if (ccode)
		exit(1);

	nuint16  volNumber;

	ccode = NWGetVolumeNumber(conn, volName, &volNumber);

	if(ccode)
	{
		printf("NWGetVolumeNumber for volume \"%s\" on connectionID 0x%08X returned status 0x%08X.\n", volName, conn, ccode );
		return (ccode);
	}

	NWFSE_VOLUME_DETAILS_BY_INFOMASK fseVolumeDetails = { 0 };
	fseVolumeDetails.ui32VolInfobyMaskSize = sizeof(fseVolumeDetails);
	nuint32 ReturnInfoMask=0;
	nuint32 volNum=volNumber;

	if ((strcmp(argv[5],"1"))==0)
		ReturnInfoMask |= VINFO_RIM_VOL_INFO64;
	else  if ((strcmp(argv[5],"2"))==0)
		ReturnInfoMask |= VINFO_RIM_VOL_NAME;
	else if ((strcmp(argv[5],"3"))==0)
		ReturnInfoMask |= VINFO_RIM_VOL_INFO64 |VINFO_RIM_VOL_NAME;

	ccode = NWGetVolumeDetailsByInfoMask( conn, volNum, 0, ReturnInfoMask , &fseVolumeDetails );

	if(ccode)
	{
		printf("NWGetVolumeDetailsByInfoMask for volume number %u on connectionID 0x%08X failed. Status=0x%08X.\n", volNum, conn, ccode );
		return (ccode);
	}

	if(ReturnInfoMask & VINFO_RIM_VOL_NAME)
	{
		nuint8  volLen;
		nuint8  volName[NW_MAX_VOLUME_NAME_LEN];

		volLen=fseVolumeDetails.VolumeNameDetails.VolNameLength;

		if ( ( volLen + 1 ) > sizeof( volName ) )
			return (NWCCODE) BUFFER_OVERFLOW;

		memmove ( volName, fseVolumeDetails.VolumeNameDetails.VolumeName, (nuint) volLen );
		volName[ volLen ] = '\0'; // Space for NULL guaranteed by our "+1" test above.
		printf("Volume name is \"%s\" \n",volName);
	}

	if(ReturnInfoMask & VINFO_RIM_VOL_INFO64)  
	{
		nuint64 totalBlocks;
		nuint64 availableBlocks;
		nuint64 purgeableBlocks;
		nuint64 totalDirEntries;
		nuint64 availableDirEntries;
		nuint64 sectorsPerBlock;

		totalBlocks = fseVolumeDetails.volumeInfo.ui64VolSizeInClusters;

		availableBlocks =  ( fseVolumeDetails.volumeInfo.ui64AvailSubAllocSectors / fseVolumeDetails.volumeInfo.ui64SectorsPerCluster ) + fseVolumeDetails.volumeInfo.ui64FreeClusters + fseVolumeDetails.volumeInfo.ui64SubAllocFreeableClusters;
		purgeableBlocks = fseVolumeDetails.volumeInfo.ui64FreeableLimboSectors / fseVolumeDetails.volumeInfo.ui64SectorsPerCluster;
		totalDirEntries = fseVolumeDetails.volumeInfo.ui64TotalDirEntries;
		availableDirEntries = fseVolumeDetails.volumeInfo.ui64UnusedDirEntries;
		sectorsPerBlock = fseVolumeDetails.volumeInfo.ui64SectorsPerCluster;

		nuint64 totalSize;
		nuint64 totalFree;
		nuint64 totalused;
		nuint64 totalDeleted;

		totalSize = totalBlocks * sectorsPerBlock * fseVolumeDetails.volumeInfo.ui64SectorSize;
		totalFree = availableBlocks * sectorsPerBlock * fseVolumeDetails.volumeInfo.ui64SectorSize;
		totalDeleted = purgeableBlocks * sectorsPerBlock * fseVolumeDetails.volumeInfo.ui64SectorSize;
		totalused = totalSize - totalFree;

		printf("The total space on volume \"%s\" is %lu bytes \n",volName,totalSize);
		printf("The used space on volume \"%s\" is %lu bytes \n",volName,totalused);
		printf("The free space on volume \"%s\" is %lu bytes \n",volName,totalFree);
		printf("The total purgeable space on volume \"%s\" is %lu bytes \n",volName,totalDeleted);
		printf("The total number of directory entries that are available on volume \"%s\" is %lu \n",volName,totalDirEntries);
		printf("The total number of directory entries that are not in use on volume \"%s\" is %lu \n",volName,availableDirEntries);

	}

	NWDSLogout(ctxHandle);
	NWDSFreeContext(ctxHandle);
	NWCCCloseConn(conn);
	NWCallsTerm(NULL);
	return(0);
}


