/*************************************************************************

Copyright (c) 2016 Novell, Inc.  All Rights Reserved.

With respect to this file, Novell hereby grants to Developer a
royalty-free, non-exclusive license to include this sample code
and derivative binaries in its product. Novell grants to Developer
worldwide distribution rights to market, distribute or sell this
sample code file and derivative binaries as a component of
Developer's product(s).  Novell shall have no obligations to
Developer or Developer's customers with respect to this code.

DISCLAIMER:

Novell disclaims and excludes any and all express, implied, and
statutory warranties, including, without limitation, warranties
of good title, warranties against infringement, and the implied
warranties of merchantibility and fitness for a particular purpose.
Novell does not warrant that the software will satisfy customer's
requirements or that the licensed works are without defect or error
or that the operation of the software will be uninterrupted.
Novell makes no warranties respecting any technical services or
support tools provided under the agreement, and disclaims all other
warranties, including the implied warranties of merchantability and
fitness for a particular purpose. */

/**************************************************************************
setDirSpaceLimitExt.c
**************************************************************************/ 

#include <stdlib.h>
#include <stdio.h>
#include <dirent.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>

#include <nwcalls.h>
#include <nwclxcon.h>
#include <nwnet.h>
#include <nwerror.h>

#define XPLAT_NO_QUOTA 0x7FFFFFFFFFFFFFFFULL

int main(int argc, char **argv)
{       
	//privilegedUserObject is used to authenticate to the tree and should have administrative 
	//privileges to set directory quota limit                       
	char nwPath[1024], privilegedUserObject[MAX_DN_CHARS], password[32], server[NW_MAX_SERVER_NAME_LEN];
	NWRCODE ccode;
	NWCONN_HANDLE conn;
	NWDSContextHandle ctxHandle;
	char treeName[NW_MAX_TREE_NAME_LEN];

	if (argc == 6)
	{
		strcpy(server, argv[1]);
		strcpy(privilegedUserObject, argv[2]);
		strcpy(password, argv[3]);
		strcpy(nwPath, argv[4]);
	}
	else
	{
		printf("Usage:\n  setDirSpaceLimit <server> <privilegedUserObject> <password> <nwPath> <0|-1|n> \n   0 - Full Restriction \n  -1 - No Restriction \n   n - quota in bytes(1 to 0x7FFFFFFFFFFFFFFE) \n");      
		exit(1);
	}

	ccode = NWCallsInit(NULL,NULL);

	if (ccode != SUCCESS)
	{
		printf("NWCallsInit failed with code 0x%04x\n", ccode);
		exit(1);
	}

	ccode = NWCCOpenConnByName (0, server, NWCC_NAME_FORMAT_BIND, NWCC_OPEN_LICENSED, NWCC_TRAN_TYPE_WILD, &conn);
	if (ccode == SUCCESS)
	{
		printf("Opened connection\n");

		ccode = NWCCGetConnInfo(conn, NWCC_INFO_TREE_NAME, sizeof(treeName), treeName);
		if (ccode == SUCCESS )
		{
			printf("   Tree name: %s\n", treeName);

			ccode = NWDSCreateContextHandle(&ctxHandle);
			if (ccode == SUCCESS)
			{
				// There must be a tree name in the context handle or a default tree name
				NWDSSetContext(ctxHandle, DCK_TREE_NAME, treeName);

				if (NWDSCanDSAuthenticate(ctxHandle) != TRUE)
				{
					NWDSSetContext(ctxHandle, DCK_NAME_CONTEXT, "");

					ccode = NWDSLoginEx(ctxHandle, privilegedUserObject, PWD_RAW_C_STRING, password);
					if (ccode == SUCCESS || ccode == NWE_ALREADY_ATTACHED)
					{
						printf("   Logged in\n");
					}
					else
						printf("NWDSLoginEx() returned 0x%x (%d)\n", ccode, ccode);
				}
				else
				{
					ccode = NWDSAuthenticateConn(ctxHandle, conn);
					if (ccode)
						printf("NWDSAuthenticateConn() returned 0x%x\n", ccode);
				}

				NWDSFreeContext(ctxHandle);
			}
			else
				printf("NWDSCreateContextHandle() returned 0x%x (%d)\n", ccode, ccode);
		}
		else
			printf("NWCCGetConnInfo() returned 0x%x (%d)\n", ccode, ccode);
	}
	else
		printf("NWCCOpenConnByName() returned 0x%x (%d)\n", ccode, ccode);

	if (ccode)
		exit(1);

	NWDIR_HANDLE dirHandle = 0;
	NWCONN_HANDLE newConnID = 0;

	ccode = NWAllocTempNSDirHandle3(conn, 0, nwPath,NW_NS_LONG,&newConnID, &dirHandle, NW_NS_LONG);

	if ( !ccode )
	{
		// If parsing the path led us to different server (DFS junction), use different server conn.
		if ( newConnID ) 
			conn = newConnID;

		// This 64-bit space restriction API needs the restriction expressed in 4KB blocks.
		// To remove a restriction / set as unlimited, the value must be 0x7FFFFFFFFFFFFFFF.
		// To set a restriction of 0 blocks, the value must be 0.

		nuint64 uRestrictionIn4KB;
		nuint64 ui64RestrictionInBytes;

		if ((strcmp(argv[5],"-1"))==0) // Unlimited
			uRestrictionIn4KB = XPLAT_NO_QUOTA;
		else if ((strcmp(argv[5],"0"))==0) // Zero bytes allowed / fully restricted.
			uRestrictionIn4KB = 0;
		else
		{
			ui64RestrictionInBytes=atol(argv[5]);
			uRestrictionIn4KB = ui64RestrictionInBytes / 4096 ;

			// If we have a non-zero restriction value of less than 4KB, force
			// a minimim value of one 4KB block.

			if ( ui64RestrictionInBytes && !uRestrictionIn4KB )
				uRestrictionIn4KB = 1;
		}

		ccode = NWSetDirSpaceLimitExt( conn, dirHandle, uRestrictionIn4KB );

		if(ccode)
		{
			printf("NWSetDirSpaceLimitExt for dirHandler %u on connectionID 0x%08X failed. Status=0x%08X.\n", dirHandle, conn, ccode );
			return (ccode);
		}

		ccode = NWDeallocateDirectoryHandle(conn, dirHandle);
		if (ccode)
		{
			printf("NWDeallocateDirectoryHandle() returned 0x%x (%d)\n", ccode, ccode);
		}

		// Don't close connID, but do close newConnID if one was opened.
		if ( newConnID )
			NWCCCloseConn( newConnID );
	}
	else
	{
		printf("NWAllocTempNSDirHandle3 failed, ccode = 0x%x\n", ccode);
		return ccode;
	}

	printf("Successfully set directory quota on directory \"%s\" \n",nwPath);

	NWDSLogout(ctxHandle);
	NWDSFreeContext(ctxHandle);
	NWCCCloseConn(conn);
	NWCallsTerm(NULL);
	return(0);
}




