#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>

#include <nwcalls.h>
#include <nwclxcon.h>
#include <nwnet.h>
#include <nwerror.h>

char *strupr(char *s)
{
   char *pch = s;

   while (*pch)
   {
      *pch = toupper(*pch);
      pch++;
   }

   return s;
}


int main(int argc, char* argv[])
{				 
   char serverName[NW_MAX_SERVER_NAME_LEN];
//   char localPath[PATH_MAX], destPath[PATH_MAX];
   char localPath[1024], destPath[1024];
   char userName[MAX_DN_CHARS * MB_CUR_MAX];

   nuint32 connRef, scanIndex = 0;
   NWRCODE ccode;
	NWCONN_HANDLE hConn = 0;

//   char mapBuffer[4096];
//   char *mapBuffer;
   nptr mapBuffer;
   nuint mapBufferLen;

	if(argc > 3)
	{
		printf("Usage:\n   nwmap [/del] [<local path>[:=<dest path>]]\n");
      exit(1);
	}

   NWCallsInit(NULL, NULL);

   if (argc == 1)
   {
      char localPath[1024], dirPath[1024];
      
      mapBufferLen = sizeof(mapBuffer);
      ccode = NWGetMappedDrives(NULL, &mapBufferLen);
      if (ccode)
      {
         if (ccode == NWE_NO_MORE_ENTRIES)
            printf("No mapped drives.\n");
         else
            printf("NWGetMappedDrives() returned 0x%x\n", ccode);
         exit(1);
      }

      if ((mapBuffer = malloc(mapBufferLen)) == NULL)
      {
         printf("Error: Out of memory\n");
         exit(1);
      }

      ccode = NWGetMappedDrives(mapBuffer, &mapBufferLen);
      if (ccode)
      {
         printf("NWGetMappedDrives() returned 0x%x\n", ccode);
         exit(1);
      }

      printf("Mapped drives: \n");

      scanIndex = 0;
      while (NWEnumMappedDrives(mapBuffer, &scanIndex, &connRef,
                                sizeof(localPath), localPath,
                                sizeof(dirPath), dirPath) == 0)
      {
         printf("%d:  (%08x)  %s -> %s\n", scanIndex, connRef, localPath, dirPath);
      }

      free(mapBuffer);
      return(0);
   }
   else if (!strcasecmp(argv[1], "/del") || !strcasecmp(argv[1], "/d"))
   {
      if (!argv[2] || strstr(argv[2], ":="))
      {
         printf("Error: Bad local path\n");
         exit(1);
      }
      strncpy(localPath, argv[2], 255);

      if ((ccode = NWDeleteDriveBase2(localPath)) != 0)
         printf("Error: unable to unmap drive\n");
      return(0);
   }
   else
   {
      char *pch, *pServer;

      if ((pch = strstr(argv[1], ":=")) == NULL)
      {
         printf("Error: Missing := separator\n");
         exit(1);
      }
      *pch = '\0';
      strncpy(localPath, argv[1], 256);

      pch += 2;
      pServer = pch;

      if (!*pch)
      {
         printf("Error: Incomplete dest path\n");
         exit(1);
      }

      pch = strchr(pServer, '\\');
      if (!pch)
         pch = strchr(pServer, '/');

      if (!pch || !*(pch+1))
      {
         printf("Error: Incomplete dest path2\n");
         exit(1);
      }

      *pch++ = '\0';
      strcpy(serverName, strupr(pServer));
      strcpy(destPath, pch);
   }


   /*
      Steps for creating a connection to a file server and mapping a drive.
      
      1. Create connection.to file server.
      
      2. Authenticate connection.

         A. Call NWCCGetConnInfo() to see if connection is already authenticated.
            If so, then we are done; map the drive (step 3).

         B. Find/create identity to file server's tree.

            Call NWDSCanDSAuthenticate() to see if an existing identity to the tree
            exists. If there is an identity to the tree, use it to authenticate
            connection (step 2.C). If not, create a new one by calling NWDSLoginEx().
            This is what is meant by logging into the tree.
            
         C. Authenticate connection with identity.

            This is where you can finally call NWDSAuthenticateConn() to authenticate
            to connection with the new identity.
         
      3. Map drive with authenticated connection.
   */   


	/* Create a new connection to server. */
	ccode = NWCCOpenConnByName(0,
										serverName,
										NWCC_NAME_FORMAT_BIND,
										NWCC_OPEN_LICENSED,  /* license when authenticated */
										NWCC_TRAN_TYPE_WILD,
										&hConn);
	if (ccode)
		printf("NWCCOpenConnByName() returned 0x%x\n", ccode);

   /*
      If we have a connection and the connection to this server is authenticated
      use it to map the drive (step 2.A). Otherwise, authenticate it.
   */

   if (ccode == SUCCESS)
   {
      NWDSContextHandle hContext = 0;
      char     treeName[NW_MAX_TREE_NAME_LEN];
      nstr     context[MAX_DN_CHARS] = "";
      char     *pch, *password;

      /* Need to login and authenticate connection to server */

      /* Create context handle */
      ccode = NWDSCreateContextHandle(&hContext);

      if (ccode == SUCCESS)
      {
         /* Get tree name of server */
         ccode = NWCCGetConnInfo(hConn, NWCC_INFO_TREE_NAME, sizeof(treeName), treeName);
//         printf("TreeName: %s\n", treeName);
         
         if (ccode == SUCCESS)
         {
            /* Set tree name in context handle. */
            ccode = NWDSSetContext(hContext, DCK_TREE_NAME, treeName);

            if (ccode == SUCCESS)
            {
               /*
                  Do we already have a identity for this tree (logged in to tree)?
                  If so just authenticate the connection with existing identity.
                  Otherwise, create new identity for this tree (step 2.B)
               */
               if (NWDSCanDSAuthenticate(hContext) != TRUE)
               {
                  /* Get user name */
                  printf("User: ");
                  fgets(userName, sizeof(userName), stdin);
                  if ((pch = strchr(userName, '\n')))
                     *pch = '\0';

                  /* Do we have a full user name w/ context? */
                  if (userName[0] != '.')
                  {
                     /* Get default name context and set default name context in context handle. */
                     if (NWDSGetDefNameContext(hContext, sizeof(context), context) != SUCCESS)
                     {
                        printf("Context: ");
                        fgets(context, sizeof(context), stdin);
                        if ((pch = strchr(context, '\n')))
                           *pch = '\0';
                        ccode = NWDSSetContext(hContext, DCK_NAME_CONTEXT, context);
                     }
                  }

                  password = getpass("Password: ");

                  /* Create new indentity to tree (login to tree) */
                  if (ccode == SUCCESS)
                  {
                     ccode = NWDSLoginEx(hContext, userName, PWD_RAW_C_STRING, password);
                     if (ccode)
                        printf("NWDSLoginEx() returned 0x%x\n", ccode);
                  }
               }

               /* Authenticate connection with new identity (step 2.C) */
               if (ccode == SUCCESS)
               {
                  ccode = NWDSAuthenticateConn(hContext, hConn);
                  if (ccode)
                     printf("NWDSAuthenticateConn() returned 0x%x\n", ccode);
               }
            }
         }

         NWDSFreeContext(hContext);
      }
   }

   /* If we have an authenticated connection, map drive with it. */
   if (ccode == SUCCESS)
   {
      ccode = NWSetDriveBase2(localPath, hConn, 0, destPath, 0);
      if (ccode != SUCCESS)
      {
         if (ccode == NWE_DRIVE_IN_USE)
         {
            /* Delete it and try again. */
            printf("Drive already mapped. Remapping to \"%s\"\n", destPath);
            ccode = NWDeleteDriveBase2(localPath);
            if (ccode != SUCCESS)
               printf("NWDeleteDriveBase2() returned 0x%x\n", ccode);
            else
               ccode = NWSetDriveBase2(localPath, hConn, 0, destPath, 0);
         }
         else
            printf("NWSetDriveBase2() returned 0x%x\n", ccode);
      }
   }

   /* Close our connection handle. */
   if (hConn)
      NWCCCloseConn(hConn);

	return(0);

}

