#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <sys/time.h>

#include <ntypes.h>
#include <nwcalls.h>
#include <nwconnec.h>
#include <nwnet.h>
#include <nwclxcon.h>
#include <nwlocale.h>
#include <nwnamspc.h>
#include <nwdirect.h>
#include <nwfile.h>

NWCCODE NWCopyFile(NWCONN_HANDLE hConn, char *srcFile, char *destFile);
NWCCODE NWTruncateFile(NWCONN_HANDLE hConn, char *destFilePath, nuint64 size);
NWDSCCODE ReadObjectAttributes(NWDSContextHandle ctxHandle, pnstr8 objectName);

char nameBuffer[256];

char *strupr(char *s)
{
	char *pch = s;

	while (*pch)
	{
		*pch = toupper(*pch);
		pch++;
	}

	return s;
}

int main(int argc, char* argv[])
{            
	NWCONN_HANDLE     hConn = 0;
	NWDSContextHandle ctxHandle = 0;
	NWCCODE           ccode;
	char              serverName[NW_MAX_SERVER_NAME_LEN];
	char              path[512];
	nstr              treeName[NW_MAX_TREE_NAME_LEN];
	nstr              userName[MAX_DN_CHARS+1];
	nstr              serverDN[MAX_DN_CHARS+1];
	nstr              password[512];
	nuint32           connRef = 0;
	NWCCConnInfo      connInfo;
	char              buf[MAX_DN_CHARS + 1];
	int               i;
	NWFILE_HANDLE     fileHandle;
	Buf_T             *pBuf;
	nint_ptr          iterator;
	nuint8            addrBuffer[6];
	NWCCTranAddr      tranAddr;
	unsigned char     readBuffer[1024];

	if (argc < 4)
	{
		printf("Usage: net <servername> <NDS username> <password> [<netware path>]\n");
		printf("net nw65 admin.novell novell\n");

		return(1);
	}

#ifdef N_PLAT_UNIX
	printf("N_PLAT_UNIX\n");
#endif
#ifdef N_ARCH_64
	printf("N_ARCH_64\n");
#endif
#ifdef N_ARCH_32
	printf("N_ARCH_32\n");
#endif

	strcpy(serverName, argv[1]);
	strcpy(userName, argv[2]);
	strcpy(password, argv[3]);
	if (argc == 5)
		strcpy(path, argv[4]);

	if ((ccode = NWCallsInit(NULL,NULL)) != SUCCESS || (ccode = NWCLXInit(NULL, NULL)) != SUCCESS)
	{
		printf("Library init failed with code 0x%04x\n", ccode);
		return(1);
	}

	NWLsetlocale(NLC_ALL, "");

	// Iterations
	for (i = 0; i < 1; i++)
	{
		printf("** NWCCOpenConnByName -> %s\n", serverName);
		ccode = NWCCOpenConnByName(0, serverName, NWCC_NAME_FORMAT_BIND, NWCC_OPEN_LICENSED, NWCC_TRAN_TYPE_WILD, &hConn);
		if (ccode == SUCCESS)
		{
			printf("   hConn = %#x\n", hConn);

			printf("******* Interation %d\n", i);

			printf("** NWCCGetConnInfo:\n");
			if (NWDSCreateContextHandle(&ctxHandle) == SUCCESS)
			{
				// There must be a tree name in the context handle or a default tree name
				printf("** NWDSLoginEx:\n");
				NWCCGetConnInfo(hConn, NWCC_INFO_TREE_NAME, sizeof(treeName), treeName);
				NWDSSetContext(ctxHandle, DCK_TREE_NAME, treeName);
				NWDSSetContext(ctxHandle, DCK_NAME_CONTEXT, "");

				ccode = NWDSLoginEx(ctxHandle, userName, PWD_RAW_C_STRING, password);
				if (ccode == SUCCESS || ccode == NWE_ALREADY_ATTACHED)
				{
					printf("   logged in to tree: %s\n", treeName);

					printf("** NWDSCanDSAthenticate:\n");
					ccode = NWDSCanDSAuthenticate(ctxHandle);
					printf("   Can Authenticate? %d\n", ccode );

					printf("** NWDSAthenticateConn:\n");
					ccode = NWDSAuthenticateConn(ctxHandle, hConn);
					if (ccode == SUCCESS)
					{
						/*
							Read login script of user
						*/

						printf("** NWDSOpenStream\n");
						ccode = NWDSOpenStream(ctxHandle, userName, "Login Script", DS_READ_STREAM | DS_WRITE_STREAM, &fileHandle);
						if (ccode == 0)
						{
							nuint32 bytesRead;

							ccode = NWReadFile(fileHandle, 1024, &bytesRead, readBuffer);
							if (ccode == 0)
							{
								int i = 0;

								printf("[BEGIN - LOGIN SCRIPT]\n");
								for (i = 0; i < bytesRead; i++)
									printf("%c", readBuffer[i]);
								printf("\n[END - LOGIN SCRIPT]\n");
							}
							NWCloseFile(fileHandle);
						}
						else
						{
							printf("   NWDSOpenStream() returned 0x%x (%d)\n", ccode, ccode);
							if (ccode == -1)
							{
								printf("   *** Error: errno=%d (%s)\n", errno, strerror(errno));
							}
						}

						/* NWDSWhoAmI */
						printf("** NWDSWhoAmI(): \n");
						ccode = NWDSWhoAmI(ctxHandle, buf); 
						if (ccode == SUCCESS )
							printf("   %s\n", buf);
						else
							printf("   NWDSWhoAmI() returned 0x%x\n", ccode);

						/* NWDSRead */
						ReadObjectAttributes(ctxHandle, buf);

						/*
							List objects under [Root]
						*/

						printf("** NWDSListByClassAndName()\n");

						ccode = NWDSAllocBuf(2048, &pBuf);
						if (!ccode)
						{
							iterator = NO_MORE_ITERATIONS;
							do
							{
								ccode = NWDSListByClassAndName(ctxHandle, "[Root]", NULL, NULL, &iterator, pBuf);
								if (ccode == 0)
								{
									nuint32 objectCount = 0, attrCount;
									Object_Info_T objectInfo;

//									printf("%s\n", objBuf.data);
									NWDSGetObjectCount(ctxHandle, pBuf, &objectCount);
									while (objectCount-- && !ccode)
									{
										nstr8 objectName[256];

										ccode = NWDSGetObjectName(ctxHandle, pBuf, objectName, &attrCount, &objectInfo);
										printf("   %s (subord #:%d, class: \"%s\")\n",
												 objectName,
												 objectInfo.subordinateCount,
												 objectInfo.baseClass);
									}
								}
								else
								{
									printf("   NWDSListByClassAndName() returned 0x%x\n", ccode);
								}
							} while (ccode == 0 && iterator != NO_MORE_ITERATIONS);

							NWDSFreeBuf(pBuf);
						}

						/* NWDSReadNDSInfo */

						printf("** NWDSReadNSDInfo()\n");

						ccode = NWDSAllocBuf(2048, &pBuf);
						if (!ccode)
						{
							NDSOSVersion_T osVersion;
							nstr8 treeName[128];
							NDSServerTime_T serverTime;
							nuint32 dsTime;
							time_t start;

							start = time(NULL);
							ccode = NWDSReadNDSInfo(hConn, 0x1F0FFF, pBuf);
							printf ("NWDSReadNDSInfo() returned 0x%x (time = %d)\n", ccode, (int)(time(NULL)-start));
							if (!ccode)
							{
								if (NWDSGetNDSInfo(ctxHandle, pBuf, DSPING_OS_VERSION, &osVersion) == 0)
									printf("   DSPING OS Version: %d.%d.%d\n",
											 osVersion.major, osVersion.minor, osVersion.revision);
								if (NWDSGetNDSInfo(ctxHandle, pBuf, DSPING_SAP_NAME, treeName) == 0)
									printf("   DSPING SAP Name: %s\n", treeName);
								if (NWDSGetNDSInfo(ctxHandle, pBuf, DSPING_TREE_NAME, treeName) == 0)
									printf("   DSPING Tree Name: %s\n", treeName);
								if (NWDSGetNDSInfo(ctxHandle, pBuf, DSPING_DS_TIME, &dsTime) == 0)
									printf("   DSPING DS Time: %d\n", dsTime);
								if (NWDSGetNDSInfo(ctxHandle, pBuf, DSPING_SERVER_TIME, &serverTime) == 0)
								{
									printf("   DSPING Server Time:\n");
									printf("       Up Time: %d\n", serverTime.upTime);
									printf("       Agent Load Time: %d\n", serverTime.agentLoadTime);
									printf("       Agent Open Time: %d\n", serverTime.agentOpenTime);
								}
							}

							ccode = NWDSReadNDSInfo(hConn, DSPING_SERVER_TIME | DSPING_SUPPORTED_FIELDS, pBuf);
							printf ("NWDSReadNDSInfo() returned 0x%x\n", ccode);
							if (!ccode)
							{
								if (NWDSGetNDSInfo(ctxHandle, pBuf, DSPING_SERVER_TIME, &serverTime) == 0)
								{
									printf("   DSPING Server Time:\n");
									printf("       Up Time: %d\n", serverTime.upTime);
									printf("       Agent Load Time: %d\n", serverTime.agentLoadTime);
									printf("       Agent Open Time: %d\n", serverTime.agentOpenTime);
								}
							}

							NWDSFreeBuf(pBuf);
						}

						/* Get conn info */
						printf("** NWCCGetAllConnInfo:\n");

						tranAddr.buffer = addrBuffer;
						tranAddr.len = sizeof(addrBuffer);
						connInfo.tranAddr = &tranAddr;

						ccode = NWCCGetAllConnInfo(hConn, NWCC_INFO_VERSION_3, &connInfo);
						if (ccode == SUCCESS )
						{
							printf("   auth state %d\n", connInfo.authenticationState);
							printf("   broadcastState %d\n", connInfo.broadcastState);
							printf("   connRef %08x\n", connInfo.connRef);
							printf("   treeName: %s\n", connInfo.treeName);
							printf("   connNum %u\n", connInfo.connNum);
							printf("   userID: %08x\n", connInfo.userID);
							printf("   serverName: %s\n", connInfo.serverName);
							printf("   NDS State: %d\n", connInfo.NDSState);
							printf("   maxPacketSize %u\n", connInfo.maxPacketSize);
							printf("   license state %d\n", connInfo.licenseState);
							printf("   distance %u\n", connInfo.distance);
							printf("   serverVersion  %u.%u.%u\n",
									 connInfo.serverVersion.major,
									 connInfo.serverVersion.minor,
									 connInfo.serverVersion.revision);
							if (tranAddr.type == NWCC_TRAN_TYPE_UDP ||
								 tranAddr.type == NWCC_TRAN_TYPE_TCP)
							{
								printf("   tranAddr %u.%u.%u.%u:%u\n",
										 tranAddr.buffer[2], tranAddr.buffer[3],
										 tranAddr.buffer[4], tranAddr.buffer[5],
										 tranAddr.buffer[0] << 8 | tranAddr.buffer[1]);
							}
							printf("   identityHandle %08x\n", connInfo.identityHandle);
						}
						else
						{
							printf("   NWCCGetAllConnInfo() returned 0x%x\n", ccode);
				//         FreeContext(ctxHandle, connRef);
						}

						printf("** NWCCGetConnRefInfo:\n");
						ccode = NWCCGetConnRef(hConn, &connRef);
						if (ccode == SUCCESS)
						{
							NWCCTranAddr tranAddr;
							nuint8 netAddress[6];
							nuint16 port;

							tranAddr.type = NWCC_TRAN_TYPE_TCP;
							tranAddr.len = sizeof(netAddress);
							tranAddr.buffer = netAddress;

							printf("  Query for NWCC_TRAN_TYPE_TCP\n");

							ccode = NWCCGetConnRefInfo(connRef, NWCC_INFO_TRAN_ADDR, sizeof(tranAddr), &tranAddr);
							if (ccode == SUCCESS )
							{
								port = tranAddr.buffer[0] << 8 | tranAddr.buffer[1];

								printf("  IP address: %d.%d.%d.%d:%d\n",
										 tranAddr.buffer[2],
										 tranAddr.buffer[3],
										 tranAddr.buffer[4],
										 tranAddr.buffer[5],
										 port);
							}
							else
								printf("   NWCCGetConnRefInfo() returned 0x%x\n", ccode);
						}

						/*
							Copy file test
						*/
//						if (argc == 5)
//						{
//							NWCopyFile(hConn, "sys:test\\MyFile", "sys:test\\MyFile2");
//						}

						printf("** NWDSLogout:\n");
						NWDSLogout(ctxHandle);
					}
					else
						printf("NWDSLoginEx() returned 0x%x (%d)\n", ccode, ccode);

					ccode = NWDSGetServerDN(ctxHandle, hConn, serverDN);
					if (ccode == SUCCESS)
					{
						printf("   serverDN = %s\n", serverDN);
					}

				}
				else
					printf("   NWDSAuthenticateConn() returned 0x%x\n", ccode);

				NWDSFreeContext(ctxHandle);
			}
			else
				printf("NWDSCreateContextHandle() returned 0x%x\n", ccode);

			NWCCCloseConn(hConn);
		}
		else
			printf("   NWCCOpenConnByName() returned 0x%x\n", ccode);

	}

	printf(" !!!!! DONE !!!!!\n");
   NWCLXTerm(NULL);
   NWCallsTerm(NULL);

	return 0;
}


char *FileName(char *path)
{
	char *pch;
	int len = strlen(path);

	pch = path + len;

	while (len--)
	{
		pch--;
		if (*pch == '/' || *pch == '\\' || *pch == ':')
		{
			pch++;
			break;
		}
	}
	return(pch);
}

NWCCODE NWCopyFile(NWCONN_HANDLE hConn, char *srcFilePath, char *destFilePath)
{
	char *pch;
	char srcName[256], destName[256];
	char srcPath[1024], destPath[1024];
	NWDIR_HANDLE hDir1, hDir2;
	NW_NS_OPEN openInfo;
	NWFILE_HANDLE hFile1, hFile2;
	NWCCODE ccode;
#define READ_BUFFER_SIZE  (32 * 1024)
	pnuint8 readBuffer;

	strcpy(srcPath, srcFilePath);
	strcpy(destPath, destFilePath);

	pch = FileName(srcPath);
	if (pch == srcPath)
		return NWE_PARAM_INVALID;
	strcpy(srcName, pch);
	if (*(pch-1) |= ':')
		*(pch-1) = '\0';
	else
		*pch = '\0';

	pch = FileName(destPath);
	if (pch == destPath)
		return NWE_PARAM_INVALID;
	strcpy(destName, pch);
	if (*(pch-1) |= ':')
		*(pch-1) = '\0';
	else
		*pch = '\0';

	if ((readBuffer = malloc(READ_BUFFER_SIZE)) == 0)
		return NWE_OUT_OF_HEAP_SPACE;

	/* Allocate temp dir handles */
	ccode = NWAllocTempNSDirHandle2Ext(hConn, 0, srcPath, NW_NS_LONG, &hDir1, NW_NS_LONG);
	if (ccode == SUCCESS )
	{
		ccode = NWAllocTempNSDirHandle2Ext(hConn, 0, destPath, NW_NS_LONG, &hDir2, NW_NS_LONG);
		if (ccode == SUCCESS )
		{
			/* Open file for read */
			openInfo.accessRights = AR_READ;
			openInfo.openCreateMode = OC_MODE_OPEN;
			openInfo.searchAttributes = SA_ALL;
			openInfo.createAttributes = 0;

			ccode = NWOpenNSEntryExt(hConn, hDir1, NW_NS_LONG, 0, srcName, &openInfo, &hFile1);
			if (ccode == SUCCESS )
			{
				/* Create file in directory */
				openInfo.accessRights = AR_READ | AR_WRITE;
				openInfo.openCreateMode = OC_MODE_REPLACE;
				openInfo.searchAttributes = SA_ALL;
				openInfo.createAttributes = A_NORMAL;

				printf("** NWOpenNSEntry (create):\n");
				ccode = NWOpenNSEntryExt(hConn, hDir2, NW_NS_LONG, 0, destName, &openInfo, &hFile2);
				if (ccode == SUCCESS )
				{
					nuint64 bytesToCopy, fileSize;
					nuint32 bytesRead;
					struct timeval start, stop;
					//                  struct timespec start, stop;
					int delta;

					ccode = NWGetFileSize(hFile1, &bytesToCopy);
					fileSize = bytesToCopy;

					printf("Copying \"%s\" -> \"%s\" (%u bytes)\n", srcName, destName, (nuint32)bytesToCopy);
					gettimeofday(&start, NULL);
					//                  clock_gettime(CLOCK_REALTIME, &start);

					while (!ccode && bytesToCopy)
					{
						ccode = NWReadFile(hFile1, READ_BUFFER_SIZE, &bytesRead, readBuffer);
						if (ccode == 0)
						{
							ccode = NWWriteFile(hFile2, bytesRead, readBuffer);
							if (ccode)
								printf("NWWriteFile() returned error 0x%x\n", ccode);
						}
						else
							printf("NWReadFile() returned error 0x%x\n", ccode);

						bytesToCopy -= bytesRead;
					}

					if (ccode == 0)
					{
						gettimeofday(&stop, NULL);
						//                  clock_gettime(CLOCK_REALTIME, &stop);
						delta = (int)((stop.tv_sec - start.tv_sec) * 1000 + ((stop.tv_usec - start.tv_usec) / 1000));
						if (delta == 0)
							delta = 1;	// 1 msec minimum

						printf("File copied (%u ms, %u KB/s)\n",
								 delta, (unsigned int)(fileSize / delta));
					}

					NWCloseFile(hFile2);
				}
				else
					printf("Error creating \"MyFile2\" (0x%x)\n", ccode);

				NWCloseFile(hFile1);
			}
			else
				printf("Error opening \"MyFile\" (0x%x)\n", ccode);

			NWDeallocateDirectoryHandle(hConn, hDir2); 
		}
		NWDeallocateDirectoryHandle(hConn, hDir1); 
	}
	else
	{
		printf("   NWAllocTempNSDirHandle2Ext() returned 0x%x\n", ccode);
	}

	free(readBuffer);

	return ccode;
}


NWCCODE NWTruncateFile(NWCONN_HANDLE hConn, char *destFilePath, nuint64 size)
{
	char *pch;
	char destName[256];
	char destPath[1024];
	NWDIR_HANDLE hDir1;
	NW_NS_OPEN openInfo;
	NWFILE_HANDLE hFile1;
	NWCCODE ccode;

	strcpy(destPath, destFilePath);

	pch = FileName(destPath);
	if (pch == destPath)
		return NWE_PARAM_INVALID;
	strcpy(destName, pch);
	if (*(pch-1) |= ':')
		*(pch-1) = '\0';
	else
		*pch = '\0';

	/* Allocate temp dir handles */
	ccode = NWAllocTempNSDirHandle2Ext(hConn, 0, destPath, NW_NS_LONG, &hDir1, NW_NS_LONG);
	if (ccode == SUCCESS )
	{
		/* Open file for read/write */
		openInfo.accessRights = AR_READ | AR_WRITE;
		openInfo.openCreateMode = OC_MODE_OPEN;
		openInfo.searchAttributes = SA_ALL;
		openInfo.createAttributes = 0;

		ccode = NWOpenNSEntryExt(hConn, hDir1, NW_NS_LONG, 0, destName, &openInfo, &hFile1);
		if (ccode == SUCCESS )
		{
			printf("Truncating file \"%s\" to %llu bytes\n", destName, size);
			ccode = NWSetFileSize(hFile1, size);
			if (ccode)
				printf("   NWSetFileSize() returned 0x%x\n", ccode);

			NWCloseFile(hFile1);
		}
		else
			printf("Error opening \"MyFile\" (0x%x)\n", ccode);

		NWDeallocateDirectoryHandle(hConn, hDir1); 
	}
	else
	{
		printf("   NWAllocTempNSDirHandle2Ext() returned 0x%x\n", ccode);
	}

	return ccode;
}


NWDSCCODE ReadObjectAttributes(NWDSContextHandle ctxHandle, pnstr8 objectName)
{
	NWDSCCODE   ccode;
	nint_ptr    iterator;
	Buf_T       *pBuf;

	printf("ReadObjectAttributes: \"%s\"\n", objectName);

	ccode = NWDSAllocBuf(2048, &pBuf);
	if (!ccode)
	{
		iterator = NO_MORE_ITERATIONS;
		do
		{
			ccode = NWDSRead(ctxHandle, objectName, DS_ATTRIBUTE_VALUES, TRUE, NULL, &iterator, pBuf);
			if (ccode == 0)
			{
				nuint32 attrCount;
				nstr8 attrName[MAX_SCHEMA_NAME_BYTES];

				ccode = NWDSGetAttrCount(ctxHandle, pBuf, &attrCount);
				if (ccode == 0)
				{
					nuint32 attrValCount, syntaxID, attrValSize;

					printf("   Attribute list, iteration: %p, count: %u\n", (void *)iterator, attrCount);
//                  NWDSGetAttrName(context)

					while (attrCount--)
					{
						NWDSGetAttrName(ctxHandle, pBuf, attrName, &attrValCount, &syntaxID);

						printf("   %s (id: %u, cnt: %u)\n", attrName, syntaxID, attrValCount);

						// skip attributes
						while (attrValCount--)
						{
							nptr attrVal;

							NWDSComputeAttrValSize(ctxHandle, pBuf, syntaxID, &attrValSize);

							attrVal = malloc(attrValSize);

							printf("      size %u\n", attrValSize);
							NWDSGetAttrVal(ctxHandle, pBuf, syntaxID, attrVal);

							free(attrVal);
						}
					}

				}
			}
		} while (ccode == 0 && iterator != NO_MORE_ITERATIONS);

		NWDSFreeBuf(pBuf);
	}

	return ccode;
}


