#include <stdlib.h>
#include <stdio.h>
#include <dirent.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>

#include <nwcalls.h>
#include <nwclxcon.h>
#include <nwnet.h>
#include <nwerror.h>

int main(int argc, char **argv)
{                                
	char nwPath[1024], user[32], password[32], server[NW_MAX_SERVER_NAME_LEN];
	NWRCODE ccode;
	NWCONN_HANDLE conn;
	NWDSContextHandle ctxHandle;
	char treeName[NW_MAX_TREE_NAME_LEN];

	if (argc == 5)
	{
		strcpy(server, argv[1]);
		strcpy(user, argv[2]);
		strcpy(password, argv[3]);
		strcpy(nwPath, argv[4]);
	}
	else
	{
		printf("Usage:\n  scan <server> <user> <password> <nwpath>\n");
		exit(1);
	}

	ccode = NWCallsInit(NULL,NULL);
	if (ccode != SUCCESS)
	{
		printf("NWCallsInit failed with code 0x%04x\n", ccode);
		exit(1);
	}

	ccode = NWCCOpenConnByName (0, server, NWCC_NAME_FORMAT_BIND, NWCC_OPEN_LICENSED, NWCC_TRAN_TYPE_WILD, &conn);
	if (ccode == SUCCESS)
	{
		printf("Opened connection\n");

		ccode = NWCCGetConnInfo(conn, NWCC_INFO_TREE_NAME, sizeof(treeName), treeName);
		if (ccode == SUCCESS )
		{
			printf("   Tree name: %s\n", treeName);

			ccode = NWDSCreateContextHandle(&ctxHandle);
			if (ccode == SUCCESS)
			{
				// There must be a tree name in the context handle or a default tree name
				NWDSSetContext(ctxHandle, DCK_TREE_NAME, treeName);

				if (NWDSCanDSAuthenticate(ctxHandle) != TRUE)
				{
					NWDSSetContext(ctxHandle, DCK_NAME_CONTEXT, "");

					ccode = NWDSLoginEx(ctxHandle, user, PWD_RAW_C_STRING, password);
					if (ccode == SUCCESS || ccode == NWE_ALREADY_ATTACHED)
					{
						printf("   Logged in\n");
					}
					else
						printf("NWDSLoginEx() returned 0x%x (%d)\n", ccode, ccode);
				}
				else
				{
					ccode = NWDSAuthenticateConn(ctxHandle, conn);
					if (ccode)
						printf("NWDSAuthenticateConn() returned 0x%x\n", ccode);
				}

				NWDSFreeContext(ctxHandle);
			}
			else
				printf("NWDSCreateContextHandle() returned 0x%x (%d)\n", ccode, ccode);
		}
		else
			printf("NWCCGetConnInfo() returned 0x%x (%d)\n", ccode, ccode);
	}
	else
		printf("NWCCOpenConnByName() returned 0x%x (%d)\n", ccode, ccode);

	if (ccode)
		exit(1);

	printf("Scanning \"%s\"\n", nwPath);

	NWDIR_HANDLE dirHandle;
	ccode = NWAllocTempNSDirHandle2Ext(conn, 0, nwPath, NW_NS_LONG, &dirHandle, NW_NS_LONG);
	if (ccode)
	{
		printf("NWAllocTempNSDirHandle2Ext failed, ccode = 0x%x\n", ccode);
		return 1;
	}

	SEARCH_SEQUENCE vIterator;
	vIterator.searchDirNumber = 0XFFFFFFFF;

	char* vMask= "*";

	printf("Scanning folder mask = \"%s\"\n", vMask);

	while (1)
	{
#define NUM_ENTRIES  20

		NW_ENTRY_INFO2_EXT pvEntry[NUM_ENTRIES];
		nuint8 moreEntries;
		nuint16 numReturned;
		nuint i;

		ccode = NWScanNSEntryInfoSet2Ext(conn,
													dirHandle,
													NW_NS_LONG,
													SA_ALL,
													&vIterator,
													vMask,
//                                       IM_ALMOST_ALL,
//                                       IM_EXTENDED_ALL,
//                                      IM_EXTENDED_ALL | IM_ALL,
													IM_SIZE | IM_ATTRIBUTES | IM_CREATION | \
													IM_EFECTIVERIGHTS | IM_64BIT_SIZE,
													&moreEntries,
													&numReturned,
													NUM_ENTRIES,
													pvEntry);

		if (ccode)
		{
			printf("NWScanNSEntryInfoSet2Ext failed, ccode = 0x%x\n", ccode);
			break;
		}

		for (i = 0; i < numReturned; i++)
		{
			NW_DATE date;
			NW_TIME time;

			NWUnpackTime(pvEntry[i].creationTime, &time);
			NWUnpackDate(pvEntry[i].creationDate, &date);

			printf("%02u/%02u/%04u  ", date.day, date.month, date.year);
			printf("%02u:%02u   ", time.hours, time.minutes);
			if (pvEntry[i].attributes & A_DIRECTORY)
				printf("%13s ", "<DIR>");
			else
				printf("%13Lu ", pvEntry[i].fileSize64);

			printf("(0x%04x) ", pvEntry[i].effectiveRights);
			printf("%-25s\n", pvEntry[i].entryName);

/*
			printf("   mod ID = %08x, creat ID = %08x", pvEntry[i].modifierID, pvEntry[i].creatorID);
			if (pvEntry[i].creatorID)
			{
				char objectName[MAX_DN_CHARS+1];

				ccode = NWDSMapIDToName(ctxHandle, conn, pvEntry[i].creatorID, objectName);
				if (!ccode)
				{
					printf(" (%s)\n", objectName);
				}
				else
				{
					printf("\n");
				}
			}
*/
		}

		if (!moreEntries)
			break;
	}

	NWDSLogout(ctxHandle);
	NWDSFreeContext(ctxHandle);
	NWCCCloseConn(conn);
	NWCallsTerm(NULL);
	return(0);
}

